!
! This program is free software; you can redistribute it and/or modify
! it under the terms of the GNU Lesser General Public License as published
! by the Free Software Foundation; either version 2.1 of the License, or
! (at your option) any later version.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU Lesser General Public License for more details.
!
! You should have received a copy of the GNU Lesser General Public
! License along with this program; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
!

System_file;

#Ifndef	WORDSIZE;
Constant	TARGET_ZCODE;
Constant	WORDSIZE	  2;
#Endif;

#Ifndef	TARGET_GLULX;
Message		"[Gtoolbar not included for non-GLULX target.]";
#Endif;

#Ifdef	TARGET_GLULX;
#Ifndef	GTOOLBAR_H;
Constant	GTOOLBAR_H;
Message		"[Including Gtoolbar.]";

! /*----------------------------------------------------------------------+
!  |      Toolbar data                                                    |
!  +----------------------------------------------------------------------*/

! Toolbar release version number.
Constant GTB_VERSION_NUMBER	  $0100;

! General toolbar colors.
Constant GTB_BACKGROUND		  $DCDCDC;
Constant GTB_LIGHT_SHADE	  $FFFFFF;
Constant GTB_DARK_SHADE		  $9A9A9A;

! Button press time in mS.
Constant GTB_PRESS_TIMEOUT	  250;

! Icon counts -- 29 for normal use, and 26 user-definable ('a'-'z').
Constant GTB_INTERNAL_ICONS	  29;
Constant GTB_USER_ICONS		  26;
Constant GTB_TOTAL_ICONS	  GTB_INTERNAL_ICONS + GTB_USER_ICONS;
Constant GTB_FIRST_USER_ICON	  GTB_INTERNAL_ICONS + 1;

! Maximum number of icons that may be displayed at a time.
Constant GTB_MAXIMUM_ICONS	  50;

! Toolbar graphical window, null if none presented, and window rock.
Global	gtb_toolbarwin		= 0;
Constant GTB_TOOLBARWIN_ROCK	  5001;

! Toolbar visibility -- normal, or collapsed.
Global	gtb_visible		= true;

! Note of any current pressed toolbar icon id.
Global	gtb_pressed		= 0;

! Toolbar icons descriptor list.
Array	gtb_default_iconlist	string ">^NO|LIZG|XTRPCD|SEAQ";
Global	gtb_iconlist		= gtb_default_iconlist;

! Toolbar icon ids and locations -- 5 entries for each icon displayed.
Array	gtb_coords		table	 5 * GTB_MAXIMUM_ICONS;

! Substituted strings -- '_' indicates overwrite, '<' prepend.
Array	gtb_s_north		string "_Nord";
Array	gtb_s_south		string "_Sud";
Array	gtb_s_east		string "_Est";
Array	gtb_s_west		string "_Ouest";
Array	gtb_s_northwest		string "_Nordouest";
Array	gtb_s_northeast		string "_Nordest";
Array	gtb_s_southwest		string "_Sudouest";
Array	gtb_s_southeast		string "_Sudest";
Array	gtb_s_up		string "_Haut";
Array	gtb_s_down		string "_Bas";
Array	gtb_s_in		string "_Dedans";
Array	gtb_s_out		string "_Dehors";
Array	gtb_s_look		string "_Regarder";
Array	gtb_s_inventory		string "_Inventaire";
Array	gtb_s_wait		string "<Attendre";
Array	gtb_s_again		string "_Encore";
Array	gtb_s_examine		string "<Examine";
Array	gtb_s_take		string "<Prendre";
Array	gtb_s_drop		string "<Laisser";
Array	gtb_s_open		string "<Ouvrir";
Array	gtb_s_close		string "<Fermer";
Array	gtb_s_read		string "<Lire";
Array	gtb_s_save		string "_Save";
Array	gtb_s_restore		string "_Restore";
Array	gtb_s_restart		string "_Recommencer";
Array	gtb_s_quit		string "_Quitter";

! String substitutions by icon id -- 0 is null substitution.
Array	gtb_iconstrings		table
	0 0 0						! separators
	gtb_s_northwest	gtb_s_north	gtb_s_northeast	! compass rose
	gtb_s_west			gtb_s_east
	gtb_s_southwest	gtb_s_south	gtb_s_southeast
	gtb_s_up	gtb_s_down			! up/down arrows
	gtb_s_in	gtb_s_out			! in/out
	gtb_s_look	gtb_s_inventory			! general
	gtb_s_wait	gtb_s_again
	gtb_s_examine					! item specific
	gtb_s_take	gtb_s_drop
	gtb_s_open	gtb_s_close
	gtb_s_read
	gtb_s_save	gtb_s_restore	gtb_s_restart	! game controls
	gtb_s_quit
	0 0 0 0 0 0 0 0 0 0 0 0 0			! user icons
	0 0 0 0 0 0 0 0 0 0 0 0 0;

! General purpose Glk arguments.
Array	gtb_glkargs		--> 8;

! /*----------------------------------------------------------------------+
!  |      General toolbar functions                                       |
!  +----------------------------------------------------------------------*/

! Display a modest 3-d effect around a region.
[ gtb_outline3d lowered x y w h
	tls brs;

	! Decide shading based on whether the region is lowered or raised.
	if (lowered) {
		tls = GTB_DARK_SHADE; brs = GTB_LIGHT_SHADE;
	} else {
		tls = GTB_LIGHT_SHADE; brs = GTB_DARK_SHADE;
	}

	! Fill areas above, left, below, and right with shading.
	glk ($EA, gtb_toolbarwin, tls, x, y, w, 1);
	glk ($EA, gtb_toolbarwin, tls, x, y, 1, h);
	glk ($EA, gtb_toolbarwin, brs, x + 2, y + h - 1, w - 2, 1);
	glk ($EA, gtb_toolbarwin, brs, x + w - 1, y + 2, 1, w - 2);
];

! Display an icon, and note its coordinates for mouse press checks.
[ gtb_icon pic x y w h index
	i;

	! Refuse, silently, if not enough coordinate array entries.
	if (index + 4 > gtb_coords-->0)
		return index;

	! Render, with a cheap button press effect if pressed.
	if (pic == gtb_pressed) {
		glk ($E1, gtb_toolbarwin, pic, x + 1, y + 1);	! draw image
		gtb_outline3d (true, x, y, w, h);
	}
	else
		glk ($E1, gtb_toolbarwin, pic, x, y);		! draw image

	! Note button coordinates.
	i = index;						! coords index
	gtb_coords-->i++ = pic;
	gtb_coords-->i++ = x;     gtb_coords-->i++ = y;
	gtb_coords-->i++ = x + w; gtb_coords-->i++ = y + h;

	! Return next index to use.
	return i;
];

! Paint in the toolbar.
[ gtb_paint
	x y w h ww wh i c id pic;

	glk ($EB, gtb_toolbarwin, GTB_BACKGROUND);	! set background color
	glk ($2A, gtb_toolbarwin);			! clear

	! Clear all current icon coordinates.
	for (i = 1: i <= gtb_coords-->0: i++)
		gtb_coords-->i = 0;

	! Start icon coord entries at 1.
	i = 1;

	! If collapsed, paint only the horizontal control separator.
	if (~~gtb_visible) {
		glk ($E0, 2, gtb_glkargs, gtb_glkargs + WORDSIZE);
		w = gtb_glkargs-->0; h = gtb_glkargs-->1;! get width and height
		x = 2; y = 0;		i = gtb_icon ( 2, x, y, w, h, i);
		return;
	}

	! Paint a small 3-d effect.
	glk ($25, gtb_toolbarwin, gtb_glkargs, gtb_glkargs + WORDSIZE);
	ww = gtb_glkargs-->0; wh = gtb_glkargs-->1;	! get window size
	gtb_outline3d (false, 0, 0, ww, wh);

	! Start with control separator.
	glk ($E0, 1, gtb_glkargs, gtb_glkargs + WORDSIZE);
	w = gtb_glkargs-->0; h = gtb_glkargs-->1;	! get width and height
	x = 2; y = 2;		i = gtb_icon ( 1, x, y, w, h, i);
	x = x + w + 2;

	! Paint icons according to the descriptor list.
	for (c = 1: c <= gtb_iconlist->0: c++) {
		switch (gtb_iconlist->c) {

		'>':					! Compass rose
			x = x + 2;
			glk ($E0,  4, gtb_glkargs, gtb_glkargs + WORDSIZE);
			w = gtb_glkargs-->0; h = gtb_glkargs-->1;
					i = gtb_icon ( 4, x, y, w, h, i);
			y = y + h;
			glk ($E0,  7, gtb_glkargs, gtb_glkargs + WORDSIZE);
			w = gtb_glkargs-->0; h = gtb_glkargs-->1;
					i = gtb_icon ( 7, x, y, w, h, i);
			y = y + h;
			glk ($E0,  9, gtb_glkargs, gtb_glkargs + WORDSIZE);
			w = gtb_glkargs-->0; h = gtb_glkargs-->1;
					i = gtb_icon ( 9, x, y, w, h, i);
			x = x + w;
			glk ($E0,  5, gtb_glkargs, gtb_glkargs + WORDSIZE);
			w = gtb_glkargs-->0; h = gtb_glkargs-->1;
			y = 2;		i = gtb_icon ( 5, x, y, w, h, i);
			y = y + h;
			glk ($E0, 10, gtb_glkargs, gtb_glkargs + WORDSIZE);
			w = gtb_glkargs-->0; h = gtb_glkargs-->1;
					i = gtb_icon (10, x, y, w, h, i);
			x = x + w;
			glk ($E0,  6, gtb_glkargs, gtb_glkargs + WORDSIZE);
			w = gtb_glkargs-->0; h = gtb_glkargs-->1;
			y = 2;		i = gtb_icon ( 6, x, y, w, h, i);
			y = y + h;
			glk ($E0,  8, gtb_glkargs, gtb_glkargs + WORDSIZE);
			w = gtb_glkargs-->0; h = gtb_glkargs-->1;
					i = gtb_icon ( 8, x, y, w, h, i);
			y = y + h;
			glk ($E0, 11, gtb_glkargs, gtb_glkargs + WORDSIZE);
			w = gtb_glkargs-->0; h = gtb_glkargs-->1;
					i = gtb_icon (11, x, y, w, h, i);
			x = x + w + 2;
			y = 2;
			continue;

		'^':					! Up/down
			x = x + 2;
			glk ($E0, 12, gtb_glkargs, gtb_glkargs + WORDSIZE);
			w = gtb_glkargs-->0; h = gtb_glkargs-->1;
					i = gtb_icon (12, x, y, w, h, i);
			y = y + h;
			glk ($E0, 13, gtb_glkargs, gtb_glkargs + WORDSIZE);
			w = gtb_glkargs-->0; h = gtb_glkargs-->1;
					i = gtb_icon (13, x, y, w, h, i);
			x = x + w + 2;
			y = 2;
			continue;

		'|','!':				! Separator.
			x = x + 2;
			glk ($E0, 3, gtb_glkargs, gtb_glkargs + WORDSIZE);
			w = gtb_glkargs-->0; h = gtb_glkargs-->1;
					i = gtb_icon ( 3, x, y, w, h, i);
			x = x + w + 2;
			continue;

		'N':	pic = 14;			! In
		'O':	pic = 15;			! Out
		'L':	pic = 16;			! Look
		'I':	pic = 17;			! Inventory
		'Z':	pic = 18;			! Wait
		'G':	pic = 19;			! Again
		'X':	pic = 20;			! Examine
		'T':	pic = 21;			! Take
		'R':	pic = 22;			! Drop
		'P':	pic = 23;			! Open
		'C':	pic = 24;			! Close
		'D':	pic = 25;			! Read
		'S':	pic = 26;			! Save
		'E':	pic = 27;			! Restore
		'A':	pic = 28;			! Restart
		'Q':	pic = 29;			! Quit

		'a' to 'z':				! user defined
			id = gtb_iconlist->i;
			pic = id - 'a' + GTB_FIRST_USER_ICON;

		default:				! unknown list entry
			continue;
		}

		! Paint the icon identified above.
		glk ($E0, pic, gtb_glkargs, gtb_glkargs + WORDSIZE);
		w = gtb_glkargs-->0; h = gtb_glkargs-->1;
					i = gtb_icon (pic, x, y, w, h, i);
		x = x + w;
	}
];

! Set toolbar visibility, and resize the window if necessary.
[ gtb_visibility v
	h p;

	! Find required height from the separator.
	gtb_visible = v;
	if (gtb_visible) {
		glk ($E0, 1, 0, gtb_glkargs);
		h = gtb_glkargs-->0 + 4;		! get height, add 4
	} else {
		glk ($E0, 2, 0, gtb_glkargs);
		h = gtb_glkargs-->0;			! get height
	}

	! Resize to this height.
	p = glk ($29, gtb_toolbarwin);			! get parent
	glk ($26, p, $12, h, 0);			! resize

	! Paint the new appearance.
	gtb_paint ();
];

! Synchronize toolbar visibility with window height, following restart.
[ gtb_synchronize_visibility
	h wh;

	! Get current toolbar window height.
	glk ($25, gtb_toolbarwin, 0, gtb_glkargs);
	wh = gtb_glkargs-->0;				! get window height

	! Get expected window height when collapsed.
	glk ($E0, 2, 0, gtb_glkargs);
	h = gtb_glkargs-->0;				! get height

	! Set visibility flag depending on current window height, repaint.
	gtb_visible = (wh > h);
	gtb_paint ();
];

! Look for mouse event in a toolbar icon, return icon id.
[ gtb_icon_click event
	i x y;

	! Get mouse coordinates.
	x = event-->2; y = event-->3;

	! Check coordinates table for a match, stopping either at the table
	! end, or on the first empty entry.
	for (i = 1: i <= gtb_coords-->0 && gtb_coords-->i ~= 0: i = i + 5) {

		! Check if this event is inside the saved coordinates.
		if (x >= gtb_coords-->(i + 1) && y >= gtb_coords-->(i + 2)
		    && x <= gtb_coords-->(i + 3) && y <= gtb_coords-->(i + 4)) {
			return gtb_coords-->i;
		}
	}

	! No match.
	return 0;
];

! Create the toolbar if doable and not yet done.
[ gtb_initialize
	h;

	! Check Glk library capabilities.
	if (gtb_toolbarwin == 0
			&& glk ($04, 6, 0)		! graphics
			&& glk ($04, 4, 5)		! mouse graphics events
			&& glk ($04, 5, 0)) {		! timer events

		! Discern initial toolbar height from the vertical separator.
		glk ($E0, 1, 0, gtb_glkargs);
		h = gtb_glkargs-->0;			! get height

		! Split main window to create toolbar at the top.
		gtb_toolbarwin = glk ($23, gg_mainwin, $12, h + 4, 5,
							GTB_TOOLBARWIN_ROCK);
		if (gtb_toolbarwin == 0)
			return;

		! Paint according to the current visibility flag.
		gtb_visibility (gtb_visible);

		! Request mouse events for the toolbar window.
		glk ($D4, gtb_toolbarwin);		! request mouse event
	}
];

! Echo a buffer as if new input.
[ gtb_echo a_buffer
	i;

	print ">";
	glk ($86, 8);					! set input style

	! Echo the overwritten buffer as if new input.
	for (i = WORDSIZE: i < a_buffer-->0 + WORDSIZE: i++) {
		print (char) a_buffer->i;
	}

	glk ($86, 0);					! set normal style
	print "^";
];

! Substitute an icon string into an input buffer.
[ gtb_overwrite subst a_buffer
	i;

	! Cancel any partial line event and simply ditch it.
	glk ($D1, gg_mainwin, 0);			! cancel line event

	! Place the substitution string into the buffer.
	for (i = 2: i <= subst->0: i++) {
		a_buffer->(WORDSIZE + i - 2) = subst->i;
	}
	a_buffer-->0 = subst->0 - 1;

	! Echo the buffer.
	gtb_echo (a_buffer);

	! Always return true.
	return true;
];

! Limited string compare, checks start of string.
[ gtb_startswith subst a_buffer len
	s i;

	! Skip leading spaces in buffer; false if nothing left.
	s = 0;
	while (s < len && a_buffer->(WORDSIZE + s) == ' ') s++;
	if (s == len)
		return false;

	! Compare characters up the end of substitute string, ignore case.
	for (i = 2: i <= subst->0: i++) {
		if (s + i - 2 >= len
			|| glk ($A1, a_buffer->(WORDSIZE + s + i - 2))
						~= glk ($A1, subst->i))
			return false;
	}
	if (s + i - 2 < len
			&& a_buffer->(WORDSIZE + s + i - 2) ~= ' ')
		return false;

	! Return matched.
	return true;
];

! Only slightly smart prepend of an icon string to an input buffer.
[ gtb_prepend subst a_buffer
	len slen found i str;

	! Cancel any partial line event but retain data.
	glk ($D1, gg_mainwin, gtb_glkargs);		! cancel line event
	if (gtb_glkargs-->0 ~= 3)
		return;					! no line request(?)

	len = gtb_glkargs-->2;				! buffered data len
	a_buffer-->0 = len;

	! Separate cases of data entered so far, and nothing so far.
	if (len > 0) {

		! See if buffer seems to contain a prependable toolbar string.
		found = false;
		for (i = 1: i <= gtb_iconstrings-->0: i++) {
			str = gtb_iconstrings-->i;
			if (str->1 == '<'
				    && gtb_startswith (str, a_buffer, len)) {
				found = true;
				break;
			}
		}

		! If it looks like it does, reissue the line request.
		if (found) {
			print ">";
			glk ($D0, gg_mainwin,		! request line event
					a_buffer + WORDSIZE,
					INPUT_BUFFER_LEN - WORDSIZE, len);
			return false;
		}

		! Prepend line so far with the substitute string, if it fits.
		slen = subst->0 - 1;
		if (len + slen + 1 < INPUT_BUFFER_LEN - WORDSIZE) {
			for (i = len - 1: i >= 0: i--) {
				a_buffer->(WORDSIZE + i + slen + 1)
						= a_buffer->(WORDSIZE + i);
			}
			for (i = 2: i <= subst->0: i++) {
				a_buffer->(WORDSIZE + i - 2) = subst->i;
			}
			a_buffer->(WORDSIZE + i - 2) = ' '; i++;
			a_buffer-->0 = a_buffer-->0 + slen + 1;
		}

		! Echo the buffer.
		gtb_echo (a_buffer);

		! Input considered done -- return true;
		return true;
	} else {

		! Start buffer with substitute string, and invite more input.
		slen = subst->0 - 1;
		for (i = 2: i <= subst->0: i++) {
			a_buffer->(WORDSIZE + i - 2) = subst->i;
		}
		a_buffer->(WORDSIZE + i - 2) = ' '; i++;

		print ">";
		glk ($D0, gg_mainwin,			! request line event
				a_buffer + WORDSIZE,
				INPUT_BUFFER_LEN - WORDSIZE, slen + 1);

		! Input not considered done -- return false.
		return false;
	}
];

! Event handler for toolbar mouse events.
[ gtb_mouse_event event context
	res pic;

	! Keep rolling request for mouse events.
	glk ($D4, gtb_toolbarwin);			! request mouse event

	! Test for a click on a toolbar icon.
	pic = gtb_icon_click (event);

	! Offer the game a chance to handle the toolbar event.
	res = GtbToolbarEvent (pic, 0, context);
	switch (res) {
	-1:						! game handled totally
		return;
	 2:						! game handled
		if (pic > 3) {
			gtb_pressed = pic; gtb_paint ();
			glk ($D6, GTB_PRESS_TIMEOUT);	! short delay callback
		}
		return;
	}

	! Handle special icon ids: none, collapse/reinstate, and spacer.
	switch (pic) {
	0:	return;					! no icon

	1:	gtb_visibility (false);			! collapse toolbar
		return;

	2:	gtb_visibility (true);			! reinstate toolbar
		return;

	3:	return;					! null separator
	}

	! If the call came from the KeyboardPrimitive context, and if there
	! is a string associated with the icon, note id and request callback.
	if (gtb_iconstrings-->pic ~= 0
			&& context == 0) {
		gtb_pressed = pic; gtb_paint ();
		glk ($D6, GTB_PRESS_TIMEOUT);		! short delay callback
	}
];

! Event handler for timer events.
[ gtb_timer_event arg
	pic str res;

	! Do nothing if no timer event is expected.
	if (gtb_pressed == 0)
		return false;

	! Note the pressed icon, and cancel press display.
	pic = gtb_pressed;
	gtb_pressed = 0; gtb_paint ();
	glk ($D6, 0);					! cancel timer

	! Offer the game a chance to handle the toolbar timeout.
	res = GtbToolbarEvent (pic, 1, arg);
	switch (res) {
	-1, 2:						! game handled totally
		return false;
	}

	! Obtain the substitution string for the pressed icon, and do no
	! more if it indicates a null string.
	str = gtb_iconstrings-->pic;
	if (str == 0)
		return false;

	! Substitute the string into the buffer.
	switch (str->1) {
	'<':						! buffer prepend
		res = gtb_prepend (str, arg);
		GtbToolbarEvent (pic, 2, arg);
		return res;

	'_':						! buffer overwrite
		res = gtb_overwrite (str, arg);
		GtbToolbarEvent (pic, 2, arg);
		return res;

	default:					! strings table error
		return false;
	}
];

! /*----------------------------------------------------------------------+
!  |      Entry point functions                                           |
!  +----------------------------------------------------------------------*/

! Glk event handler entry point.
[ HandleGlkEvent event context abortres
	done res;

	! If no toolbar window is available, ignore the event.
	done = false;
	if (gtb_toolbarwin ~= 0) {

		! Note events only from either the KeyboardPrimitive[0] or
		! KeyCharPrimitive[1] functions.
		switch (context) {
		0, 1:						! primitive
			switch (event-->0) {
			4:					! mouse
				if (event-->1 == gtb_toolbarwin)
					gtb_mouse_event (event, context);
			1:					! timer
				done = gtb_timer_event (abortres);
			5, 6:					! arrange/redraw
				gtb_paint ();
			}
		}
	}

	! Chain entry point.
	res = GtbHandleGlkEvent (event, context, abortres);

	! Return 2, special code to exit KeyboardPrimitive, if we expect line
	! input to terminate, otherwise return chained entry result.
	if (done)
		return 2;
	return res;
];

! Glk window initialization entry point.
[ InitGlkWindow winrock
	res;

	! Intercept status window creation.  Ideally, we'd create the toolbar
	! on main window creation, but in practice since the status window is
	! always created after the main window, and that's the one we want to
	! split, it's a little easier to leave it until here.
	switch (winrock) {
	GG_STATUSWIN_ROCK:
		! Create a toolbar at mainwin top, allowing game first dibs.
		res = GtbInitGlkWindow (GTB_TOOLBARWIN_ROCK);
		if (res == 0)
			gtb_initialize ();
	}

	! Chain entry point.
	return GtbInitGlkWindow (winrock);
];

! Glk object identification entry point.
[ IdentifyGlkObject phase type ref rock;

	! Reassert control over the toolbar following restart.
	switch (phase) {
	0:							! start
		gtb_toolbarwin = 0;
	1:							! pass ref
		switch (type) {
		0:						! window ref
			if (rock == GTB_TOOLBARWIN_ROCK) {
				gtb_toolbarwin = ref;
				return;
			}
		}
	2:							! tidy up
		if (gtb_toolbarwin ~= 0) {
			gtb_synchronize_visibility ();
		}
	}

	! Chain entry point.
	return GtbIdentifyGlkObject (phase, type, ref, rock);
];

! Inform after-prompt entry point.
[ AfterPrompt;

	! The Xglk library loses mouse events inside its save/restore
	! file dialogs.  This means that we can find ourselves without
	! an outstanding mouse request on the toolbar, which makes it
	! stop working.  To get around this, here we'll request mouse
	! events on each game prompt issued, relying on the fact that,
	! unlike char and line events, the Glk library doesn't complain
	! if a mouse event is already requested.  And, unfortunately,
	! this means that we have to take control of this entry point...
	if (gtb_toolbarwin ~= 0)
		glk ($D4, gtb_toolbarwin);		! request mouse event

	! Chain entry point.
	return GtbAfterPrompt ();
];

! /*----------------------------------------------------------------------+
!  |      Entry point stub functions                                      |
!  +----------------------------------------------------------------------*/

! Chaining entry point stubs.
#Ifndef Replace_GtbIdentifyGlkObject;
#Stub GtbIdentifyGlkObject	4;
#Endif;
#Ifndef Replace_GtbHandleGlkEvent;
#Stub GtbHandleGlkEvent		2;
#Endif;
#Ifndef Replace_GtbInitGlkWindow;
#Stub GtbInitGlkWindow		1;
#Endif;
#Ifndef Replace_GtbAfterPrompt;
#Stub GtbAfterPrompt		0;
#Endif;

! Private entry point stub.
#Ifndef Replace_GtbToolbarEvent;
#Stub GtbToolbarEvent		3;
#Endif;

! /*----------------------------------------------------------------------+
!  |      Programming interface functions                                 |
!  +----------------------------------------------------------------------*/

! Return toolbar code version.
[ gtb_version;

	return GTB_VERSION_NUMBER;
];

! Query if a toolbar is available.
[ gtb_is_available;

	return (gtb_toolbarwin ~= 0);
];

! Query and set toolbar visibility.
[ gtb_get_visible;

	return (gtb_toolbarwin ~= 0 && gtb_visible);
];
[ gtb_set_visible flag;

	! If displayed, update, otherwise just save flag.
	if (gtb_toolbarwin ~= 0)
		gtb_visibility (flag);
	else
		gtb_visible = flag;
];

! Query and set icon display list.
[ gtb_get_icon_list;

	return gtb_iconlist;
];
[ gtb_set_icon_list list;

	! If null list passed in, set default, otherwise set list.
	if (list == 0)
		gtb_iconlist = gtb_default_iconlist;
	else
		gtb_iconlist = list;
	if (gtb_toolbarwin ~= 0 && gtb_visible)
		gtb_paint ();
];

! Query and set icon strings.
[ gtb_get_icon_string id index					! varargs
	pic;

	! Handle direct picture resource id retrievals.
	if (id >= 4 && id <= gtb_iconstrings-->0)
		return gtb_iconstrings-->id;

	! Handle retrievals that give icon list id instead.
	switch (id) {
		'>':					! Compass rose
			switch (index) {
			1 to 8:	pic = 4 + index - 1;
			default: return -1;
			}

		'^':					! Up/down
			switch (index) {
			1, 2:	pic = 12 + index - 1;
			default: return -1;
			}

		'N':	pic = 14;			! In
		'O':	pic = 15;			! Out
		'L':	pic = 16;			! Look
		'I':	pic = 17;			! Inventory
		'Z':	pic = 18;			! Wait
		'G':	pic = 19;			! Again
		'X':	pic = 20;			! Examine
		'T':	pic = 21;			! Take
		'R':	pic = 22;			! Drop
		'P':	pic = 23;			! Open
		'C':	pic = 24;			! Close
		'D':	pic = 25;			! Read
		'S':	pic = 26;			! Save
		'E':	pic = 27;			! Restore
		'A':	pic = 28;			! Restart
		'Q':	pic = 29;			! Quit

		'a' to 'z':				! User defined
			pic = id - 'a' + GTB_FIRST_USER_ICON;

		default:
			return -1;
	}
	return gtb_iconstrings-->pic;
];

[ gtb_set_icon_string id str str2 str3 str4 str5 str6 str7 str8	! varargs
	pic;

	! Verify that the first icon string, if present, has valid format.
	if (str ~= 0 && str->1 ~= '_' or '<')
		return false;

	! Handle direct picture resource id settings.
	if (id >= 4 && id <= gtb_iconstrings-->0) {
		gtb_iconstrings-->id = str;
		return true;
	}

	! Handle settings that give icon list id instead.
	switch (id) {
		'>':					! Compass rose
			if ((str2 ~= 0 && str2->1 ~= '_' or '<')
					|| (str3 ~= 0 && str3->1 ~= '_' or '<')
					|| (str4 ~= 0 && str4->1 ~= '_' or '<')
					|| (str5 ~= 0 && str5->1 ~= '_' or '<')
					|| (str6 ~= 0 && str6->1 ~= '_' or '<')
					|| (str7 ~= 0 && str7->1 ~= '_' or '<')
					|| (str8 ~= 0 && str8->1 ~= '_' or '<'))
				return false;
			gtb_iconstrings-->4  = str;
			gtb_iconstrings-->5  = str2;
			gtb_iconstrings-->6  = str3;
			gtb_iconstrings-->7  = str4;
			gtb_iconstrings-->8  = str5;
			gtb_iconstrings-->9  = str6;
			gtb_iconstrings-->10 = str7;
			gtb_iconstrings-->11 = str8;
			return true;

		'^':					! Up/down
			if (str2 ~= 0 && str2->1 ~= '_' or '<')
				return false;
			gtb_iconstrings-->12 = str;
			gtb_iconstrings-->13 = str2;
			return true;

		'N':	pic = 14;			! In
		'O':	pic = 15;			! Out
		'L':	pic = 16;			! Look
		'I':	pic = 17;			! Inventory
		'Z':	pic = 18;			! Wait
		'G':	pic = 19;			! Again
		'X':	pic = 20;			! Examine
		'T':	pic = 21;			! Take
		'R':	pic = 22;			! Drop
		'P':	pic = 23;			! Open
		'C':	pic = 24;			! Close
		'D':	pic = 25;			! Read
		'S':	pic = 26;			! Save
		'E':	pic = 27;			! Restore
		'A':	pic = 28;			! Restart
		'Q':	pic = 29;			! Quit

		'a' to 'z':				! User defined
			pic = id - 'a' + GTB_FIRST_USER_ICON;

		default:
			return false;
	}
	gtb_iconstrings-->pic = str;
	return true;
];

! Create a new toolbar if none yet done.
[ gtb_create;

	! If no toolbar create one.  Note that this isn't the same as when
	! it's done by InitGlkWindow.  Since the toolbar is a split of the
	! main window, if done here, it will appear below the status line.
	! This appears to be unavoidable given Inform library architecture.
	if (gtb_toolbarwin == 0)
		gtb_initialize ();

	return (gtb_toolbarwin ~= 0);
];

! Remove the toolbar from the display.
[ gtb_destroy;

	! Close any open toolbar window.
	if (gtb_toolbarwin ~= 0) {
		glk ($24, gtb_toolbarwin, 0);		! close window
		gtb_toolbarwin = 0;
	}
];

#Endif;
#Endif;
